#!/usr/bin/env python
"""Make a deb package for epidermis
Should be run in a subdirectory of the project's root directory
The .py files should be under the epidermis subdirectory"""

import sys, os
import subprocess

copyright = \
"""This package was debianized by David D Lowe

Upstream author: David D Lowe

This software is copyright (c) 2008 David D Lowe

---------------- original license follows -----------------------
LICENSE
-------

Epidermis theme manager

GNU General Public License
--------------------------
Copyright (C) 2008  David D Lowe

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA."""

package = "epidermis"
version = "0.1e1"
priority = "optional"
section = "misc"
maintainer = "David D Lowe <flimm@find-me-at-ubuntuforums-dot-org-or-at-launchpad-dot-net>"
depends = "python, usplash, python-gobject, python-glade2, python-gnome2, python-gtk2 (>= 2.4), python-pexpect, coreutils"
title = "Epidermis theme manager"
description = \
""" Epidermis changes the appearance of your Ubuntu GNOME desktop in all its aspects in one click. Epidermis 'skins' change the appearance of your desktop wallpaper, Metacity windows border theme, your GTK controls theme, your icon theme, your mouse cursor theme, your GRUB bootsplash screen, your GDM login screen theme and your Usplash boot splash image.
 .
 Each of these customisations are downloaded in 'pigments' which are available from an Epidermis 'repository'.
 .
 Epidermis was written in python with pygtk.
 .
 Find more information at http://epidermis.tuxfamily.org and https://launchpad.net/epidermis/
 .
 This version is the an experimental version, in which repo functionality has been disabled.
""" # remember final newline

dotDesktop = \
"""[Desktop Entry]
Version=1.0
Encoding=UTF-8
Name=Epidermis
Comment=Restyle your entire desktop rapidly
Generic Name=theme manager
Terminal=false
Icon=epidermis
Type=Application
Exec=epidermis
Categories=System;
StartupNotify=True
"""

blacklist0 = ("build.py", "test.py", "oldmanagepigments.py", "oldmp.py")

def main():
    
    dp = determine_path()
    pp = os.path.join(dp[:-1*len(dp.split("/")[-1])], "epidermis")
    # pp should be the epidermis dir with all the .py files in it
    
    # check if DEBUG is False
    ff = open(os.path.join(pp, "const.py"), "r")
    debugging = False
    while True:
        ln = ff.readline()
        if ln == "" or ln is None:
            break
        if len(ln.strip().replace(" ", "")) > 7:
            if ln.strip().replace(" ", "")[:6] == "DEBUG=":
                if ln.strip().replace(" ", "")[6:10] == "True":
                    debugging = True
                    break
    ff.close()
    if debugging:
        print >> sys.stderr, "- DEBUG is set to True"
        print >> sys.stderr, "To fix this, change DEBUG to False in const.py"
    
    # check if gtkexcepthook is imported
    ff = open(os.path.join(pp, "epidermis.py"), "r")
    imported = False
    while True:
        ln = ff.readline()
        if ln == "" or ln is None:
            break
        if len(ln.strip().replace(" ", "")) > 7:
            if ln.strip().replace(" ", "")[:19] == "importgtkexcepthook":
                imported = True
                break
    ff.close()
    if not imported:
        print >> sys.stderr, "- gtkexcepthook is not imported"
        print >> sys.stderr, "To fix this, add this line to epidermis.py:"
        print >> sys.stderr, "import gtkexcepthook"
    
    if debugging or not imported:
        return
    
    # prepare directories
    subprocess.call(["rm", "-r", os.path.join(pp, "build")])
    for dir in ("DEBIAN", "usr/bin", "usr/share/applications", "usr/share/epidermis/epidermis", \
    "usr/share/epidermis/data", "usr/share/epidermis/epidermis/pigments", "usr/share/pixmaps"):
        subprocess.call(["mkdir", "-p", os.path.join(pp, "build", dir)])
        
    # copy files to usr/share/epidermis/epidermis
    for fil in os.listdir(pp):
        if fil[-3:] == ".py" or fil[-4:] == ".pyc":
            if fil not in blacklist0 and fil[:-1] not in blacklist0:
                subprocess.call(["cp", os.path.join(pp, fil), os.path.join(pp, "build/usr/share/epidermis/epidermis")])
    # copy to usr/share/epidermis/epidermis/pigments
    for fil in os.listdir(os.path.join(pp, "pigments")):
        if fil[-3:] == ".py" or fil[-4:] == ".pyc":
            subprocess.call(["cp", os.path.join(pp, "pigments", fil), os.path.join(pp, "build/usr/share/epidermis/epidermis/pigments")])
    # copy to usr/share/epidermis/data
    for fil in os.listdir(os.path.join(pp, "data")):
        if fil[-1] != "~" and fil[-3:] != "bak":
            subprocess.call(["cp", os.path.join(pp, "data", fil), os.path.join(pp, "build/usr/share/epidermis/data")])
    # move the scripts
    for fil in ("gksubash.sh", "gksued.sh"):
        subprocess.call(["mv", os.path.join(pp, "build/usr/share/epidermis/data", fil), os.path.join(pp, "build/usr/share/epidermis")])
        
    # create icon symlink
    subprocess.call(["ln", "-s", "../epidermis/data/epidermis.svg", "epidermis.svg"], cwd=os.path.join(pp, "build/usr/share/pixmaps"))
    
    # write epidermis.desktop file
    subprocess.call(["touch", os.path.join(pp, "build/usr/share/applications/epidermis.desktop")])
    ff = open(os.path.join(pp, "build/usr/share/applications/epidermis.desktop"), "w")
    ff.write(dotDesktop)
    ff.close()
    
    # make runnable link to epidermis.py
    subprocess.call(["chmod" ,"+x", os.path.join(pp, "build/usr/share/epidermis/epidermis/epidermis.py")])
    subprocess.call(["ln", "-s", "../share/epidermis/epidermis/epidermis.py", "epidermis"], cwd=os.path.join(pp, "build/usr/bin"))
    
    # from http://mail.python.org/pipermail/python-list/2000-June/037295.html
    def calcDirSize(arg, dir, files):
        for file in files:
            if os.path.isfile(os.path.join(dir, file)): # eliminates directories and symlinks
                stats = os.stat(os.path.join(dir, file))
                size = stats[6]
                arg.append(size)

    def getDirSize(dir):
        """Get directory size in kilobytes. (at least 1KB)"""
        sizes = []
        os.path.walk(dir, calcDirSize, sizes)
        total = 0
        for size in sizes:
            total = total + size
        if total > 1024:
            return int(round(total/1024.0, 2))
        else:
            return 1

    # calculate Installed-Size
    size = getDirSize(os.path.join(pp, "build"))
    
    # write files in DEBIAN
    subprocess.call(["touch", os.path.join(pp, "build/DEBIAN/copyright")])
    ff = open(os.path.join(pp, "build/DEBIAN/copyright"), "w")
    ff.write(copyright)
    ff.close()
    
    subprocess.call(["touch", os.path.join(pp, "build/DEBIAN/control")])
    ff = open(os.path.join(pp, "build/DEBIAN/control"), "w")
    ff.write("Package: " + package + "\n")
    ff.write("Version: " + version + "\n")
    ff.write("Priority: " + priority + "\n")
    ff.write("Section: " + section + "\n")
    ff.write("Installed-Size: " + str(size) + "\n")
    ff.write("Maintainer: " + maintainer + "\n")
    ff.write("Architecture: all\n")
    ff.write("Depends: " + depends + "\n")
    ff.write("Description: " + title + "\n")
    ff.write(description)
    ff.close()
    
    # build deb package
    subprocess.call(["dpkg", "--build", "build/", package + "-" + version + "_all.deb"], cwd=pp)
    
    

def determine_path ():
    """Borrowed from wxglade.py"""
    try:
        root = __file__
        if os.path.islink (root):
            root = os.path.realpath (root)
        return os.path.dirname (os.path.abspath (root))
    except Exception, ee:
        print >> sys.stderr, "I'm sorry, but something is wrong."
        print >> sys.stderr, "There is no __file__ variable. Please contact the author."
        sys.exit (1)
        raise(ee)


if __name__ == "__main__":
    main()