#!/usr/bin/env python
# Copyright 2008 David D Lowe
# Released under the GPLv2
"""Module with constant variables that are used accross several modules
in the Epidermis project

Import at the beginning of the module like this:
>>> from const import *
>>> if __name__ == "__main__":
...    set_up_const()
...    from const import *
...

The set_up_const function must be called for values like USERNAME to be correct"""

# Pigment directory shared between all users
SHARED_PIGMENT_DIR = "/usr/share/epidermis/pigments/"

# List of all recognised pigment types
PIGMENT_TYPES = ["skin", "wallpaper", "metacity", "gtk", "icons", "gnomeSplash", \
    "cursors", "grub", "gdm", "usplash"]

# Are we in debugging mode?
DEBUG = False

# Some strings
CUSTOM = "CUSTOM: " # remember to surround this string with _()
APP_NAME = "epidermis"
LOCALE_DIR = "translation"
VERSION = "0.21"
VERSION_TAG = ""

const_initialised = False

import os, sys

# Get current user's username, as printed by whoami
__pop = os.popen("whoami")
__output = __pop.readline()[:-1]
USERNAME = __output

# Cache directory for current user, MY_CACHE_HOME is usually ~/.cache/epidermis
if os.environ.has_key('XDG_CACHE_HOME'):
    CACHE_HOME = os.environ['XDG_CACHE_HOME']
else:
    if os.environ.has_key('HOME'):
        CACHE_HOME = os.path.join(os.environ['HOME'],".cache")
    else:
        print >> sys.stderr,  '$HOME environment variable could not be found, exiting'
        raise(Exception("HOME varible not found"))
        sys.exit(1)
MY_CACHE_HOME = os.path.join(CACHE_HOME, "epidermis")
# Standards: http://standards.freedesktop.org/basedir-spec/latest/ar01s03.html
if os.environ.has_key('XDG_CONFIG_HOME'):
    CONFIG_HOME = os.environ['XDG_CONFIG_HOME']
else:
    if os.environ.has_key('HOME'):
        CONFIG_HOME = os.path.join(os.environ['HOME'], ".config")
    else:
        print >> sys.stderr, '$HOME environment variable could not be found, exiting'
        sys.exit(1)
MY_CONFIG_HOME = os.path.join(CONFIG_HOME, "epidermis/")

if os.environ.has_key('XDG_DATA_HOME'):
    DATA_HOME = os.environ['XDG_DATA_HOME']
else:
    if os.environ.has_key('HOME'):
        DATA_HOME = os.path.join(os.environ['HOME'],".local/share")
    else:
        print >> sys.stderr, '$HOME environment variable could not be found, exiting'
        sys.exit()
MY_DATA_HOME = os.path.join(DATA_HOME, "epidermis/")

const_initialized = True

def set_up_const():
    raise(Exception("set_up_const is deprecated"))
    pass
    
    

if __name__ == "__main__":
    print "Don't run"

