#!/usr/bin/env python

from pigment import PigmentType, Pigment
import pigment
import os, sys
import subprocess
import gconf
from glob import glob
if __name__ == "__main__" or 'pychecker' in sys.argv[0]:
    sys.path.append("..")
import ep_exceptions
import handy
import hashlib
from gettext import gettext as _


class CursorsType(PigmentType):
    humanName = {"en":"Cursors"}
    humanPluralName = {"en":"Cursors"}
    codeName = "cursors"
    abbreviation = "cr"
    directoryName = "cursors"

    attachmentTypeHelp = _("The theme file should be the directory that bears " +
      "the name of theme, and containing a directory named cursors, it should " +
      "not contain any other directories")
        

class Cursors(Pigment):
    
    
    def __init__(self):
        Pigment.__init__(self)
        self.type = CursorsType()
        self.cursorsDirname = ""
        self.cursorsPath = ""
    
    def add_attachment(self, attachmentPath):
        if not len(self.attachments) == 0:
           raise(ep_exceptions.TooManyAttachmentsException(len(self.attachments)))
        if os.path.isabs(attachmentPath) and os.path.exists(attachmentPath):
            realPath = attachmentPath
        else:
            if os.path.exists(os.path.join("/usr/share/icons", attachmentPath.split("/")[-1], "cursors")):
                realPath = os.path.join("/usr/share/icons", attachmentPath.split("/")[-1])
            else:
                self.cursorsPath = ""
                self.cursorsDirname = ""
                raise(ep_exceptions.AttachmentNotFoundException(attachmentPath))
        self.cursorsPath = realPath
        self.cursorsDirname = attachmentPath.split("/")[-1]
        Pigment.add_attachment(self, realPath)
    
    
    def install(self, installationDirectory=None, progressCallback=None, shell=None):
        exitPlease = False
        if progressCallback is None: progressCallback = lambda progress, query: False
        def change_progress_callback(progress, query):
            if progressCallback(progress / 2, query):
                exitPlease = True
                return True
            return False
        # copy pigment information over
        Pigment.install(self, installationDirectory, change_progress_callback, shell)
        if exitPlease: return
        
        # copy cursors directory
        if progressCallback is None: progressCallback = lambda progress, query: False
        if os.path.exists(os.path.join("/usr/share/icons", self.cursorsDirname, "cursors")):
            query = pigment.QUERY_OVERWRITE_DATA
        else:
            query = None
        if progressCallback(0.75, query): return
        shell.do(["mkdir", "-p", "/usr/share/icons/%s" % self.cursorsDirname])
        shell.do(handy.join_lists(["cp", "-r"], glob(self.cursorsPath + "/*"), \
                ["/usr/share/icons/%s/" % self.cursorsDirname]))
        progressCallback(1.0, None)
            
    
    def remove(self, installationDirectory=None, progressCallback=None, shell=None):
        exitPlease = False
        def change_progress_callback(progress):
            if progressCallback(progress / 2):
                exitPlease = True
                return True
            return False
        # copy pigment information over
        Pigment.remove(self, installationDirectory, change_progress_callback, shell)
        if exitPlease: return
        
        # remove cursors dir
        #def ignore(progress): return False
        if progressCallback is None: progressCallback = lambda(progress): False
        if progressCallback(0.75): return
        shell.do(["rm", "-r", "/usr/share/icons/%s/cursors" % self.cursorsDirname])
        shell.do(["rm", "-f", "/usr/share/icons/%s/cursors.theme" % self.cursorsDirname])
        foundDir = False
        for dd in os.listdir("/usr/share/icons/%s/" % self.cursorsDirname):
            if os.path.isdir("/usr/share/icons/%s/%s" % (self.cursorsDirname, dd)):
                foundDir = True
                break
        if not foundDir:
            subprocess.call(["rm", "-r", "/usr/share/icons/%s" % self.cursorsDirname])
        progressCallback(1.0)
    
    def activate(self, progressCallback=None, shell=None):
        if progressCallback is None:
            progressCallback = lambda(a1): False
        if progressCallback(0.0): return
        client = gconf.client_get_default()
        client.set_string("/desktop/gnome/peripherals/mouse/cursor_theme", self.cursorsDirname)
        progressCallback(1.0)
    
    def read(self, filePath):
        Pigment.read(self, filePath)
        
        from xml.dom import minidom
        doc = minidom.parse(filePath)
        root = doc.getElementsByTagName("pigment")[0]

        attachmentPath = filePath[:(-1*len(filePath.split("/")[-1]))]
        self.add_attachment(os.path.join(attachmentPath, "data", pigment.get_first_text( \
            root.getElementsByTagName("dirname")[0])))
        #self.cursorsDirname = pigment.get_first_text(root.getElementsByTagName("dirname")[0])
        
        
        
    def information_xml(self):
        """Returns the XML document describing the pigment"""
        doc = Pigment.information_xml(self)
        root = doc.getElementsByTagName("pigment")[0]
        
        xmlBit = doc.createElement("dirname")
        tn = doc.createTextNode(self.cursorsDirname)
        xmlBit.appendChild(tn)
        root.appendChild(xmlBit)
        
        return doc
    
    
    def is_installed(self, installationDirectory=None):
        """Returns whether this pigment is installed
        installationDirectory: the directory in which to search for already 
        installed pigments"""
        if Pigment.is_installed(self, installationDirectory):
            if os.path.isdir("/usr/share/icons/%s/cursors.theme" % self.cursorsDirname):
                return True
        return False
    
    def is_activated(self):
        """Returns whether this pigment is applied. To check if the effects
        have been activated regardless of Epidermis' pigments, use is_meat_activated"""
        return self.is_meat_activated()
    
    def is_attachment_correct(self, attachmentIndex=None):
        if attachmentIndex is None and len(self.attachments) != 1:
            return False
        if not attachmentIndex is None:
            if attachmentIndex >= len(self.attachments):
                print >> sys.stderr, "no attachment with index ", attachmentIndex
            att = self.attachments[attachmentIndex]
        else:
            att = self.attachments[0]
        dir = att.split("/")[-1]
        if dir in ("cursors"):
            return False
        if not os.path.isdir(att):
            return False
        if not os.path.isdir(os.path.join(att,"cursors")):
            return False
        for dir in os.listdir(att):
            if os.path.isdir(dir):
                if dir != "cursors":
                    return False
        if len(os.listdir(os.path.join(att, "cursors"))) == 0:
            return False
        return Pigment.is_attachment_correct(self, attachmentIndex)
    
    def convert_from_meat(self, meat, progressCallback=None):
        # TODO: convert_from_meat
        pass
    
    def is_meat_installed(self):
        """Returns whether the meat of this pigment or similar to it is installed"""
        (md1, md2) = (hashlib.md5(), hashlib.md5())
        # FIXME: cursors is_installed
        try:
            f1 = glob(self.cursorsPath + "/*") 
            f2 = glob("/usr/share/themes/%s/*" % self.cursorsDirname)
        except:
            return False
        for mm, ff in ((md1, f1), (md2, f2)):
            for af in ff:
                try:
                    ss = file(af, "rb").read(8096)
                except:
                    return False
            if not ss:
                break
            mm.update(ss)
        return m1 == m2
        
    
    def is_meat_activated(self):
        """Returns whether the meat of this pigment or similar to it is activated"""
        (md1, md2) = (hashlib.md5(), hashlib.md5())
        client = gconf.client_get_default()
        # FIXME: meat_activated
        try:
            f1 = glob(self.cursorsPath + "/*")
            f2 = glob("/usr/share/icons/%s/*" % client.get_string("/desktop/gnome/peripherals/mouse/cursor_theme")) 
        except:
            return False
        for mm, ff in ((md1, f1), (md2, f2)):
            for af in ff:
                try:
                    ss = file(af, "rb").read(8096)
                except:
                    return False
            if not ss:
                break
            mm.update(ss)
        return m1 == m2
    
    def does_activation_require_root(self):
        """Returns whether the activation requires root rights.
        If it does, the pigment object will need a pexpect bash shell as root
        to be passed to some of its methods"""
        return False
    
    def match_filter(self, string, lang="en"):
        """Returns whether this pigment should be a result for a search for the
        string passed"""
        for ii in (self.get_human_name(lang), self.get_description(lang)):
            if string.lower() in ii.lower():
                return True
        return False
    
    def get_display_data(self, index=0):
        """Returns the object of index to be added to a gtk.ListStore, this
        could be a string, a gtk.gdk.Pixbuf or a custom widget, it should
        be rendered using type.create_cell_renderer"""
        return

def main():
    wp = Icons()
    print wp.information_xml().toxml()

if __name__ == "__main__":
    main()