#!/usr/bin/env python
# Copyright 2008 David D Lowe
# released under the GPLv2

from __future__ import division
from pigment import PigmentType, Pigment
import pigment
import os, sys
import thread
import gconf
import subprocess
from glob import glob
if __name__ == "__main__" or 'pychecker' in sys.argv[0]:
    sys.path.append("..")
import ep_exceptions
import handy
import hashlib
from gettext import gettext as _


class IconsType(PigmentType):
    humanName = {"en":"Icons"}
    humanPluralName = {"en":"Icons"}
    codeName = "icons"
    abbreviation = "ic"
    directoryName = "icons"
    
    attachmentTypeHelp = _("The theme file should be a directory that contains many " +
        "others including 32x32 and that does not contain a cursors directory")
        

class Icons(Pigment):
    
    
    def __init__(self):
        Pigment.__init__(self)
        self.type = IconsType()
        self.iconsDirname = ""
        self.iconsPath = ""
    
    def add_attachment(self, attachmentPath):
        if not len(self.attachments) == 0:
           raise(ep_exceptions.TooManyAttachmentsException(len(self.attachments)))
        if os.path.isabs(attachmentPath) and os.path.exists(attachmentPath):
            realPath = attachmentPath
        else:
            if os.path.exists(os.path.join("/usr/share/icons", attachmentPath.split("/")[-1])):
                realPath = os.path.join("/usr/share/icons", attachmentPath.split("/")[-1])
            else:
                self.iconsPath = ""
                self.iconsDirname = ""
                raise(ep_exceptions.AttachmentNotFoundException(attachmentPath))
        self.iconsPath = realPath
        self.iconsDirname = attachmentPath.split("/")[-1]
        Pigment.add_attachment(self, realPath)
    
    
    def install(self, installationDirectory=None, progressCallback=None, shell=None):
        exitPlease = False
        if progressCallback is None: progressCallback = lambda progress, query: False
        def change_progress_callback(progress, query):
            if progressCallback(progress / 2, query):
                exitPlease = True
                return True
            return False
        # copy pigment information over
        Pigment.install(self, installationDirectory, change_progress_callback, shell)
        if exitPlease: return
        
        if progressCallback is None: progressCallback = lambda progress, query: False
        query = None
        if os.path.exists(os.path.join("/usr/share/icons", self.iconsDirname)):
            for tt in os.listdir(os.path.join("/usr/share/icons/", self.iconsDirname)):
                if os.path.isdir(os.path.join("/usr/share/icons/", self.iconsDirname, tt)) and tt != "cursors":
                    query = pigment.QUERY_OVERWRITE_DATA
                    break
        if progressCallback(0.5, query): return
        shell.do(["mkdir", "-p", "/usr/share/icons/%s" % self.iconsDirname])
        shell.do(handy.join_lists(["cp", "-r"], glob(self.iconsPath + "/*"), \
                ["/usr/share/icons/%s/" % self.iconsDirname]))
        shell.do(["chown", "root:root", "-R", "/usr/share/icons/%s" % self.iconsDirname])
        shell.do(["chmod", "755", "-R", "/usr/share/icons/%s" % self.iconsDirname])
        
        alldirs = [] # save a list of all subdirectories for later
        os.path.walk("/usr/share/icons/%s" % self.iconsDirname, lambda a1, dirname, a3: alldirs.append(dirname), None)
        
        shell.do(["chmod", "644", "-R", "/usr/share/icons/%s" % self.iconsDirname])
        shell.do(["chmod", "755", "/usr/share/icons/%s" % self.iconsDirname])
        if progressCallback(0.75, None): return
        counter = 0
        count = len(alldirs)
        for dir in alldirs:
            shell.do(["chmod", "755", dir])
            if progressCallback(0.75 + (counter / count) / 4, None): return
            counter += 1

        progressCallback(1.0, None)
            
    
    def remove(self, installationDirectory=None, progressCallback=None,shell=None):
        exitPlease = False
        def change_progress_callback(progress):
            if progressCallback(progress / 2):
                exitPlease = True
                return True
            return False
        # copy pigment information over
        Pigment.remove(self, installationDirectory, change_progress_callback, shell)
        if exitPlease: return
        
        #def ignore(progress): return False
        if progressCallback is None: progressCallback = lambda(progress): False
        if progressCallback(0.75): return
        if os.path.isdir("/usr/share/icons/%s/cursors" % self.iconsDirname):
            for dd in os.listdir("/usr/share/icons/%s/" % self.iconsDirname):
                if os.path.isdir("/usr/share/icons/%s/%s" % (self.iconsDirname, dd)) and dd != "cursors":
                    shell.do(["rm", "-r", "/usr/share/icons/%s/%s" % (self.iconsDirname, dd)])
        else:
            shell.do(["rm", "-r", "/usr/share/icons/%s" % self.iconsDirname])
        progressCallback(1.0)
    
    def activate(self, progressCallback=None, shell=None):
        if progressCallback is None:
            progressCallback = lambda(a1): False
        if progressCallback(0.0): return
        client = gconf.client_get_default()
        client.set_string("/desktop/gnome/interface/icon_theme", self.iconsDirname)
        progressCallback(1.0)
   
    def read(self, filePath):
        Pigment.read(self, filePath)
        
        from xml.dom import minidom
        doc = minidom.parse(filePath)
        root = doc.getElementsByTagName("pigment")[0]
        
        attachmentPath = filePath[:(-1*len(filePath.split("/")[-1]))]
        self.add_attachment(os.path.join(attachmentPath, "data", pigment.get_first_text( \
            root.getElementsByTagName("dirname")[0])))
                
    def information_xml(self):
        """Returns the XML document describing the pigment"""
        doc = Pigment.information_xml(self)
        root = doc.getElementsByTagName("pigment")[0]
        
        xmlBit = doc.createElement("dirname")
        tn = doc.createTextNode(self.iconsDirname)
        xmlBit.appendChild(tn)
        root.appendChild(xmlBit)
        
        return doc
    
    
    def is_installed(self, installationDirectory=None):
        """Returns whether this pigment is installed
        installationDirectory: the directory in which to search for already 
        installed pigments"""
        if Pigment.is_installed(self, installationDirectory):
            if os.path.isdir("/usr/share/icons/%s" % self.iconsDirname):
                return True
        return False
    
    def is_activated(self):
        """Returns whether this pigment is applied. To check if the effects
        have been activated regardless of Epidermis' pigments, use is_meat_activated"""
        return self.is_meat_activated()

    def is_attachment_correct(self, attachmentIndex=None):
        if attachmentIndex is None and len(self.attachments) != 1:
            return False
        if not attachmentIndex is None:
            if attachmentIndex >= len(self.attachments):
                print >> sys.stderr, "no attachment with index ", attachmentIndex
            att = self.attachments[attachmentIndex]
        else:
            att = self.attachments[0]
        if not os.path.isdir(att):
            return False
        if os.path.isdir(os.path.join(att, "cursors")):
            return False
        if not os.path.isdir(os.path.join(att, "32x32")):
            return False
        return Pigment.is_attachment_correct(self, attachmentIndex)
    
    def convert_from_meat(self, meat, progressCallback=None):
        # TODO: convert_from_meat
        pass
    
    def is_meat_installed(self):
        """Returns whether the meat of this pigment or similar to it is installed"""
        (md1, md2) = (hashlib.md5(), hashlib.md5())
        try:
            f1 = glob(self.iconsPath + "/*") 
            f2 = glob("/usr/share/icons/%s/*" % self.iconsDirname)
            # FIXME: icons
            if "cursors.theme" in f2:
                f2.remove("cursors.theme")
            if "cursors" in f2:
                f2.remove("cursors")
        except:
            return False
        for mm, ff in ((md1, f1), (md2, f2)):
            for af in ff:
                try:
                    ss = file(af, "rb").read(8096)
                except:
                    return False
            if not ss:
                break
            mm.update(ss)
        return m1 == m2
        
    
    def is_meat_activated(self):
        """Returns whether the meat of this pigment or similar to it is activated"""
        (md1, md2) = (hashlib.md5(), hashlib.md5())
        client = gconf.client_get_default()
        try:
            f1 = glob(self.iconsPath + "/*")
            f2 = glob("/usr/share/icons/%s/*" % client.get_string("/desktop/gnome/interface/icon_theme")) 
        except:
            return False
        for mm, ff in ((md1, f1), (md2, f2)):
            for af in ff:
                try:
                    ss = file(af, "rb").read(8096)
                except:
                    return False
            if not ss:
                break
            mm.update(ss)
        return m1 == m2
    
    def does_activation_require_root(self):
        """Returns whether the activation requires root rights.
        If it does, the pigment object will need a pexpect bash shell as root
        to be passed to some of its methods"""
        return False
    
    def match_filter(self, string, lang="en"):
        """Returns whether this pigment should be a result for a search for the
        string passed"""
        for ii in (self.get_human_name(lang), self.get_description(lang)):
            if string.lower() in ii.lower():
                return True
        return False
    
    def get_display_data(self, index=0):
        """Returns the object of index to be added to a gtk.ListStore, this
        could be a string, a gtk.gdk.Pixbuf or a custom widget, it should
        be rendered using type.create_cell_renderer"""
        return

def main():
    wp = Icons()
    print wp.information_xml().toxml()

if __name__ == "__main__":
    main()